"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("../config");
const utils_1 = require("../utils");
const webpack = require("webpack");
const path_1 = require("path");
const chalk_1 = require("chalk");
const WebpackDevServer = require("webpack-dev-server");
const ramda_1 = require("ramda");
const nodemon = require("nodemon");
const opn = require("opn");
const log = console.log;
module.exports = opts => {
    const userConfig = utils_1.getUserConfig();
    const config = config_1.default("development", userConfig, opts.d, opts.port);
    config.mode = "development";
    if (utils_1.isBackEndConfig(userConfig.mode)) {
        webpack(config).watch({}, ramda_1.once(error => {
            if (error) {
                console.log(chalk_1.default.red(error));
                process.exit(1);
                return;
            }
            nodemon({
                script: path_1.join(utils_1.rootPath, "./dist/index.js"),
                watch: path_1.join(utils_1.rootPath, "./dist/**/*")
            }).on("quit", process.exit);
        }));
    }
    else {
        const port = opts.port;
        const open = opts.open;
        const hostname = "0.0.0.0";
        const options = {
            contentBase: path_1.join(utils_1.rootPath, "./assets"),
            open,
            openPage: "",
            overlay: true,
            inline: true,
            host: hostname,
            quiet: true,
            port: opts.port,
            hot: true,
            historyApiFallback: true,
            proxy: userConfig && userConfig.devServer && userConfig.devServer.proxy
        };
        config.devServer = options;
        const server = new WebpackDevServer(webpack(config), options);
        server.listen(port, hostname, error => {
            if (error) {
                server.close();
                log(chalk_1.default.red(error.message));
                process.exit(1);
            }
            if (open) {
                const url = `http://localhost:${port}`;
                opn(url);
            }
        });
    }
};
//# sourceMappingURL=dev.js.map