"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../utils");
const fs_1 = require("fs");
const path_1 = require("path");
const globby = require("globby");
const mkdirp = require("mkdirp");
const rimraf = require("rimraf");
const sh = require("shelljs");
const chalk_1 = require("chalk");
const writeFile = (fullpath, content) => {
    const sections = fullpath.split("/");
    sections.pop();
    const path = sections.join("/");
    mkdirp.sync(path);
    fs_1.writeFileSync(fullpath, content);
};
const copyTemplate = (name, dirname, { mode, d, force }) => __awaiter(this, void 0, void 0, function* () {
    const pkg = {
        name,
        version: "0.0.1",
        private: true,
        scripts: {
            dev: "tp dev",
            build: "tp build",
            test: "tp test"
        }
    };
    const paths = yield globby(path_1.join(__dirname, `../../templates/${mode}/**/*`));
    const files = paths.map((path) => {
        const splittedName = path.split(path_1.join(__dirname, `../../templates/${mode}/`));
        return {
            name: splittedName[splittedName.length - 1],
            content: fs_1.readFileSync(path)
        };
    });
    if (d) {
        console.log(paths);
        console.log(files);
    }
    if (!paths || !paths.length) {
        console.log(chalk_1.default.red("Template not exists."));
        process.exit(1);
        return;
    }
    if (force) {
        rimraf.sync(dirname);
    }
    if (fs_1.existsSync(dirname)) {
        console.log(chalk_1.default.red("Dirname already exists."));
        console.log(`use ${chalk_1.default.blue("--force")} to delete it and create a new one.`);
        process.exit(1);
        return;
    }
    mkdirp.sync(dirname);
    files.forEach(file => {
        writeFile(path_1.join(utils_1.rootPath, name, file.name), file.content);
    });
    writeFile(path_1.join(utils_1.rootPath, name, "package.json"), JSON.stringify(pkg, null, 2));
});
const installPackages = (dirname, mode) => {
    const hasYarn = sh.which("yarn");
    const install = hasYarn ? "yarn add" : "npm install --save";
    const devInstall = hasYarn ? "yarn add -D" : "npm install --save-dev";
    const dependencies = ["@babel/runtime"];
    const devDependencies = [
        "typepack",
        "@types/node",
        "@types/jest",
        "@babel/core",
        "@babel/preset-env",
        "@babel/plugin-transform-runtime"
    ];
    if (mode === "react") {
        const reactDeps = ["react", "react-dom"];
        const reactDevDeps = ["@types/react", "@types/react-dom"];
        dependencies.push(...reactDeps);
        devDependencies.push(...reactDevDeps);
    }
    sh.cd(dirname);
    if (sh.exec(`${install} ${dependencies.join(" ")}`).code !== 0 ||
        sh.exec(`${devInstall} ${devDependencies.join(" ")}`).code !== 0) {
        sh.echo("Error: failed to install needed modules");
        sh.exit(1);
    }
};
module.exports = (name, opts) => __awaiter(this, void 0, void 0, function* () {
    const dirname = path_1.join(utils_1.rootPath, name);
    yield copyTemplate(name, dirname, opts);
    installPackages(dirname, opts.mode);
});
//# sourceMappingURL=init.js.map